#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2020 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
import warnings

from bx_django_utils.test_utils.html_assertion import HtmlAssertionMixin
from django.contrib.auth.models import User
from django.test import TestCase
from reversion.models import Revision, Version

# Needs to import admin module to register all models via CompareVersionAdmin/VersionAdmin
from reversion_compare_project.utils.fixtures import Fixtures


class BaseTestCase(HtmlAssertionMixin, TestCase):
    def setUp(self):
        super().setUp()

        self.fixtures = Fixtures()
        self.user = User.objects.create_superuser("Test User", "nobody@local.intranet", "no password")
        self.client.force_login(self.user)

    def tearDown(self):
        super().tearDown()

        Revision.objects.all().delete()
        Version.objects.all().delete()

    def assertContainsHtml(self, response, *args):
        warnings.warn('Use assert_html_parts()', DeprecationWarning)
        self.assert_html_parts(response, parts=args)

    def assertNotContainsHtml(self, response, *args):
        warnings.warn('Use assert_parts_not_in_html()', DeprecationWarning)
        self.assert_parts_not_in_html(response, parts=args)



from django.apps import AppConfig as BaseAppConfig


class AppConfig(BaseAppConfig):
    name = 'reversion_compare'
    verbose_name = 'reversion_compare'

    def ready(self):
        import reversion_compare.checks  # noqa



# flake8: noqa: E405
"""
    Settings used to run tests
"""
from reversion_compare_project.settings.prod import *  # noqa


# _____________________________________________________________________________
# Manage Django Project

INSTALLED_APPS.append('manage_django_project')

# _____________________________________________________________________________


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

SECRET_KEY = 'No individual secret for tests ;)'

DEBUG = True

# Speedup tests by change the Password hasher:
PASSWORD_HASHERS = ('django.contrib.auth.hashers.MD5PasswordHasher',)

# _____________________________________________________________________________

# Skip download map via geotiler in reversion_compare.gpx_tools.gpxpy2map.generate_map
MAP_DOWNLOAD = False


# All tests should use django-override-storage!
# Set root to not existing path, so that wrong tests will fail:
STATIC_ROOT = '/not/exists/static/'
MEDIA_ROOT = '/not/exists/media/'



"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2020 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from reversion import create_revision, is_registered, set_comment
from reversion.models import Revision, Version

from reversion_compare_project.models import Person, Pet
from reversion_compare_project.utils.fixtures import Fixtures

# Needs to import admin module to register all models via CompareVersionAdmin/VersionAdmin
from reversion_compare_project.utils.test_cases import BaseTestCase


class PersonPetModelTest(BaseTestCase):
    """
    unittests that used:
        reversion_compare_test_app.models.Person
        reversion_compare_test_app.models.Pet

    Person & Pet are registered with the follow information, so that
    related data would be also stored in django-reversion

    see "Advanced model registration" here:
        https://github.com/etianen/django-reversion/wiki/Low-level-API
    """

    def setUp(self):
        super().setUp()

        fixtures = Fixtures(verbose=False)
        # fixtures = Fixtures(verbose=True)
        self.pet1, self.pet2, self.person = fixtures.create_PersonPet_data()

        queryset = Version.objects.get_for_object(self.person)
        self.version_ids = queryset.values_list("pk", flat=True)

    def test_initial_state(self):
        self.assertTrue(is_registered(Pet))
        self.assertTrue(is_registered(Person))

        self.assertEqual(Pet.objects.count(), 3)

        self.assertEqual(Version.objects.get_for_object(self.pet1).count(), 2)
        self.assertEqual(Revision.objects.all().count(), 2)

    def test_select_compare(self):
        response = self.client.get(f"/en/admin/reversion_compare_project/person/{self.person.pk}/history/")
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{self.version_ids[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{self.version_ids[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[1]:d}" />',
        )

    def test_diff(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/person/{self.person.pk}/history/compare/",
            data={"version_id2": self.version_ids[0], "version_id1": self.version_ids[1]},
        )
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            """
                <div class="module">
                    <p class="highlight">
                        <del>would be changed pet</del> &rarr; <ins>Is changed pet</ins><br />
                        <del>- would be deleted pet</del><br />
                        <del>- would be removed pet</del><br />
                        always the same pet<br />
                    </p>
                </div>
            """,
            "<blockquote>version 2: change follow related pets.</blockquote>",  # edit comment
        )
        self.assertNotContainsHtml(
            response,
            "<h3>name</h3>",  # person name doesn't changed
            #
            # All fields are under reversion control:
            ' not under reversion control.' 'class="follow"',
        )

    def test_add_m2m(self):
        with create_revision():
            # Add a pet to person:
            new_pet = Pet.objects.create(name="added pet")
            self.person.pets.add(new_pet)
            self.person.save()
            set_comment("version 3: add a pet")

        self.assertEqual(Revision.objects.all().count(), 3)
        self.assertEqual(Version.objects.all().count(), 12)

        queryset = Version.objects.get_for_object(self.person)
        version_ids = queryset.values_list("pk", flat=True)
        self.assertEqual(len(version_ids), 3)

        response = self.client.get(f"/en/admin/reversion_compare_project/person/{self.person.pk}/history/")
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{version_ids[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{version_ids[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{version_ids[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{version_ids[1]:d}" />',
            f'<input type="radio" name="version_id2" value="{version_ids[2]:d}" />',
            f'<input type="radio" name="version_id2" value="{version_ids[2]:d}" />',
        )

        response = self.client.get(
            f"/en/admin/reversion_compare_project/person/{self.person.pk}/history/compare/",
            data={"version_id2": version_ids[0], "version_id1": version_ids[1]},
        )
        # debug_response(response) # from django-tools

        self.assertContainsHtml(
            response,
            """
            <div class="module">
                <p class="highlight">
                    <ins>+ added pet</ins><br />
                    Is changed pet<br />
                    always the same pet<br />
                </p>
            </div>
            """,
            "<blockquote>version 3: add a pet</blockquote>",  # edit comment
        )
        self.assertNotContainsHtml(
            response,
            "<h3>name</h3>",  # person name doesn't changed
            #
            # All fields are under reversion control:
            ' not under reversion control.' 'class="follow"',
        )

    def test_m2m_not_changed(self):
        with create_revision():
            self.person.name = "David"
            self.person.save()
            set_comment("version 3: change the name")

        self.assertEqual(Revision.objects.all().count(), 3)
        self.assertEqual(Version.objects.all().count(), 11)

        queryset = Version.objects.get_for_object(self.person)
        version_ids = queryset.values_list("pk", flat=True)
        self.assertEqual(len(version_ids), 3)

        response = self.client.get(f"/en/admin/reversion_compare_project/person/{self.person.pk}/history/")
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{version_ids[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{version_ids[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{version_ids[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{version_ids[1]:d}" />',
            f'<input type="radio" name="version_id2" value="{version_ids[2]:d}" />',
            f'<input type="radio" name="version_id2" value="{version_ids[2]:d}" />',
        )

        response = self.client.get(
            f"/en/admin/reversion_compare_project/person/{self.person.pk}/history/compare/",
            data={"version_id2": version_ids[0], "version_id1": version_ids[1]},
        )
        # debug_response(response) # from django-tools

        self.assertContainsHtml(
            response,
            """
            <pre class="highlight">
            <del>- Dave</del>
            <ins>+ David</ins>
            </pre>
            """,
            "<blockquote>version 3: change the name</blockquote>",  # edit comment
        )
        # FIXME: All fields are under reversion control
        # self.assertNotContainsHtml(response,
        #     "pet",
        #     #
        #     # All fields are under reversion control:
        #     ' not under reversion control.'
        #     'class="follow"',
        # )



import logging

from reversion.models import Version

from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class MigrationModelTestCase(BaseTestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        fixtures = Fixtures(verbose=False)
        cls.instance = fixtures.create_MigrationModel_data()

        queryset = Version.objects.get_for_object(cls.instance)
        cls.version_ids = list(queryset.values_list("pk", flat=True))

    def test_compare_raw(self):
        assert self.version_ids == [4, 3, 2, 1]
        assert self.instance.pk == 1

        response = self.client.get('/en/admin/reversion_compare_project/migrationmodel/1/history/')
        self.assertEqual(response.status_code, 200, response)
        self.assertTemplateUsed(response, 'reversion-compare/object_history.html')
        self.assert_html_parts(
            response,
            parts=(
                '<input type="submit" value="compare">',
                '<td>Migration state 2 - version 4</td>',
                '<td>Migration state 2 - version 3</td>',
                '<td>Migration state 1 - version 2</td>',
                '<td>Migration state 1 - version 1</td>',
            ),
        )

        with self.assertLogs('reversion_compare', level=logging.WARNING) as logs:
            response = self.client.get(
                '/en/admin/reversion_compare_project/migrationmodel/1/history/compare/',
                data={'version_id2': 3, 'version_id1': 2},
            )
            self.assertEqual(response.status_code, 200, response)
            self.assertTemplateUsed(response, 'reversion-compare/compare_raw.html')
            self.assert_html_parts(
                response,
                parts=(
                    '<blockquote>Migration state 2 - version 3</blockquote>',
                    '<del>2</del>',
                    '<ins>3</ins>',
                    'Not a number 2',
                    'Lorem ipsum dolor sit amet,',
                    'Revert to the old version will probably not work!',
                ),
            )
        error_msg = logs.output[0]
        assert 'Fallback compare caused by:' in error_msg
        assert 'incompatible version data' in error_msg



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Test history compare CBV

    :copyleft: 2012-2017 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from django.db import connection
from django.test.utils import CaptureQueriesContext
from reversion import is_registered
from reversion.models import Version

from reversion_compare_project.models import SimpleModel
from reversion_compare_project.utils.db_queries import print_db_queries
from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class CBViewTest(BaseTestCase):
    """
    unittests for testing reversion_compare.views.HistoryCompareDetailView

    Tests for the basic functions.
    """

    def setUp(self):
        super().setUp()
        fixtures = Fixtures(verbose=False)
        self.item1, self.item2 = fixtures.create_Simple_data()

        queryset = Version.objects.get_for_object(self.item1)
        self.version_ids1 = queryset.values_list("pk", flat=True)

        queryset = Version.objects.get_for_object(self.item2)
        self.version_ids2 = queryset.values_list("pk", flat=True)

    def test_initial_state(self):
        self.assertTrue(is_registered(SimpleModel))

        self.assertEqual(SimpleModel.objects.count(), 2)
        self.assertEqual(SimpleModel.objects.all()[0].text, "version two")

        self.assertEqual(Version.objects.get_for_object(self.item1).count(), 2)
        self.assertEqual(list(self.version_ids1), [2, 1])

        self.assertEqual(list(self.version_ids1), [2, 1])
        self.assertEqual(list(self.version_ids2), [7, 6, 5, 4, 3])

    def test_admin_demo_links(self):
        response = self.client.get('/en/admin/')
        self.assertContainsHtml(
            response,
            '<h2>HistoryCompareDetailView Examples:</h2>',
            '<a href="/en/test_view/1/">/en/test_view/1/</a>',
            '<a href="/en/test_view/2/">/en/test_view/2/</a>',
        )

    def assert_select_compare1(self, response):
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{self.version_ids1[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids1[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{self.version_ids1[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids1[1]:d}" />',
        )

    def test_select_compare1(self):
        response = self.client.get(f"/en/test_view/{self.item1.pk}/")
        self.assert_select_compare1(response)

    def test_select_compare1_queries(self):
        with CaptureQueriesContext(connection) as queries:
            response = self.client.get(f"/en/test_view/{self.item1.pk}/")
            self.assert_select_compare1(response)

        print_db_queries(queries.captured_queries)
        # total queries....: 9
        # unique queries...: 6
        # duplicate queries: 3

        self.assertLess(len(queries.captured_queries), 7 + 2 + 1)  # real+buffer+login

    def test_select_compare2(self):
        response = self.client.get(f"/en/test_view/{self.item2.pk}/")
        for i in range(4):
            if i == 0:
                comment = f"create v{i:d}"
            else:
                comment = f"change to v{i:d}"

            self.assertContainsHtml(response, f"<td>{comment}</td>", '<input type="submit" value="compare">')

    def assert_select_compare_and_diff(self, response):
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{self.version_ids1[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids1[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{self.version_ids1[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids1[1]:d}" />',
        )
        self.assertContainsHtml(
            response,
            "<del>- version one</del>",
            "<ins>+ version two</ins>",
            "<blockquote>simply change the CharField text.</blockquote>",  # edit comment
        )

    def test_select_compare_and_diff(self):
        response = self.client.get(
            f"/en/test_view/{self.item1.pk}/",
            data={"version_id2": self.version_ids1[0], "version_id1": self.version_ids1[1]},
        )
        self.assert_select_compare_and_diff(response)

    def test_select_compare_and_diff_queries(self):
        with CaptureQueriesContext(connection) as queries:
            response = self.client.get(
                f"/en/test_view/{self.item1.pk}/",
                data={"version_id2": self.version_ids1[0], "version_id1": self.version_ids1[1]},
            )
            self.assert_select_compare_and_diff(response)

        print_db_queries(queries.captured_queries)
        # total queries....: 17
        # unique queries...: 11
        # duplicate queries: 6
        self.assertLess(len(queries.captured_queries), 15 + 2 + 1)  # real+buffer+login

    def test_prev_next_buttons(self):
        base_url = f"/en/test_view/{self.item2.pk}/"
        for i in range(4):
            # IDs: 3,4,5,6
            id1 = i + 3
            id2 = i + 4
            response = self.client.get(base_url, data={"version_id2": id2, "version_id1": id1})
            self.assertContainsHtml(
                response,
                f"<del>- v{i:d}</del>",
                f"<ins>+ v{i+1:d}</ins>",
                f"<blockquote>change to v{i+1:d}</blockquote>",
            )

            next = f'<a href="?version_id1={i + 4}&amp;version_id2={i + 5}">next &rsaquo;</a>'
            prev = f'<a href="?version_id1={i + 2}&amp;version_id2={i + 3}">&lsaquo; previous</a>'

            if i == 0:
                self.assertNotContains(response, "previous")
                self.assertContains(response, "next")
                self.assertContainsHtml(response, next)
            elif i == 3:
                self.assertContains(response, "previous")
                self.assertNotContains(response, "next")
                self.assertContainsHtml(response, prev)
            else:
                self.assertContains(response, "previous")
                self.assertContains(response, "next")
                self.assertContainsHtml(response, prev, next)



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2020 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from django.db import connection
from django.test.utils import CaptureQueriesContext
from reversion import is_registered, revisions, unregister
from reversion.models import Revision, Version

from reversion_compare_project.models import Car, Factory, Person
from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class FactoryCarReverseRelationModelTest(BaseTestCase):
    """
    unittests that used:
        reversion_compare_test_app.models.Factory
        reversion_compare_test_app.models.Car

    Factory & Car would be registered only in admin.py
    so no relation data would be stored
    """

    def setUp(self):
        unregister(Person)
        unregister(Car)
        unregister(Factory)
        revisions.register(Factory, follow=["building_ptr", "cars", "workers"])
        revisions.register(Car)
        revisions.register(Person, follow=["pets"])
        super().setUp()

        fixtures = Fixtures(verbose=False)
        self.factory = fixtures.create_Factory_reverse_relation_data()
        queryset = Version.objects.get_for_object(self.factory)
        self.version_ids = queryset.values_list("pk", flat=True)

    def test_initial_state(self):
        self.assertTrue(is_registered(Factory))
        self.assertTrue(is_registered(Car))
        self.assertTrue(is_registered(Person))
        self.assertEqual(Revision.objects.all().count(), 3)
        self.assertEqual(len(self.version_ids), 3)
        self.assertEqual(Version.objects.all().count(), 19)

    def test_select_compare(self):
        response = self.client.get(f"/en/admin/reversion_compare_project/factory/{self.factory.pk}/history/")
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{self.version_ids[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{self.version_ids[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[1]:d}" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[2]:d}" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[2]:d}" />',
        )

    def assert_diff1(self, response):
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            """
            <p class="highlight">
                <ins>+ Bob Bobertson</ins><br />
            </p>
            """,
            """
            <div class="module">
                <p class="highlight">
                    <del>- motor-car three from factory one supplier(s): </del>  &rarr; Deleted<br />
                    <ins>+ motor-car four from factory one supplier(s): </ins><br />
                    motor-car one from factory one supplier(s): <br />
                </p>
            </div>
            """,
            # edit comment:
            "<blockquote>version 2: discontinued car-three, add car-four, add Bob the worker</blockquote>",
        )

    def test_diff1(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/factory/{self.factory.pk}/history/compare/",
            data={"version_id2": self.version_ids[1], "version_id1": self.version_ids[2]},
        )
        self.assert_diff1(response)

    def test_select_compare1_queries(self):
        with CaptureQueriesContext(connection) as queries:
            response = self.client.get(
                f"/en/admin/reversion_compare_project/factory/{self.factory.pk}/history/compare/",
                data={"version_id2": self.version_ids[1], "version_id1": self.version_ids[2]},
            )
            self.assert_diff1(response)

        # print_db_queries(queries.captured_queries)
        # total queries....: 37
        # unique queries...: 28
        # duplicate queries: 9

        self.assertLess(len(queries.captured_queries), 37 + 2)  # real+buffer



from pathlib import Path


print("Use settings:", __file__)

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = Path(__file__).parent

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [str(Path(BASE_DIR, 'templates'))],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


MIDDLEWARE = (
    "debug_toolbar.middleware.DebugToolbarMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    # WARNING:
    # This will 'disable' the authentication, because
    # the default user will always be logged in.
    "reversion_compare_project.middleware.AlwaysLoggedInAsSuperUser",
    "django.contrib.messages.middleware.MessageMiddleware",
    "reversion.middleware.RevisionMiddleware",
)

# Username/password for reversion_compare_project.middleware.AlwaysLoggedInAsSuperUser:
DEFAULT_USERNAME = "AutoLoginUser"
DEFAULT_USERPASS = "no password needed!"

SECRET_KEY = "unittests-fake-key"

LANGUAGE_CODE = 'en-us'
USE_I18N = True
USE_L10N = True
USE_TZ = True
LOCALE_PATHS = (BASE_DIR.parent / 'reversion_compare' / 'locale',)

PASSWORD_HASHERS = ["django.contrib.auth.hashers.MD5PasswordHasher"]  # Speeding up the tests

SITE_ID = 1

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django.contrib.admin",
    "django.contrib.sessions",
    "django.contrib.messages",
    "debug_toolbar",
    "reversion",
    "reversion_compare",
    "reversion_compare_project",
]

ROOT_URLCONF = "reversion_compare_project.urls"

STATIC_URL = '/static/'
STATIC_ROOT = str(BASE_DIR.parent / 'static')

MEDIA_URL = '/media/'
MEDIA_ROOT = str(BASE_DIR.parent / 'media')

INTERNAL_IPS = [
    '127.0.0.1',
]

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'


DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": str(Path(BASE_DIR.parent, "test_project_db.sqlite3")),
        # 'NAME': ":memory:",
        'OPTIONS': {
            # https://docs.djangoproject.com/en/2.2/ref/databases/#database-is-locked-errors
            'timeout': 20,
        },
    }
}

DEBUG = True

# add reversion models to django admin:
ADD_REVERSION_ADMIN = True

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(asctime)s %(levelname)s %(name)s %(filename)s:%(lineno)d %(message)s',
        },
    },
    'handlers': {
        'console': {'class': 'logging.StreamHandler', 'formatter': 'verbose'},
    },
    'loggers': {
        'django': {'handlers': ['console'], 'level': 'INFO', 'propagate': False},
        'django_tools': {'handlers': ['console'], 'level': 'INFO', 'propagate': False},
        'revision_compare': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
        'revision_compare_tests': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': False},
    },
}



from django import template
from django.urls import reverse
from django.utils.safestring import mark_safe

from reversion_compare_project.models import SimpleModel


register = template.Library()


@register.simple_tag
def demo_links():
    pks = SimpleModel.objects.values_list('pk', flat=True)[:3]
    links = []
    for pk in pks:
        url = reverse('test_view', kwargs={'pk': pk})
        links.append(f'<li><a href="{url}">{url}</a></li>')
    links = ''.join(links)
    return mark_safe(f'<ul>{links}</ul>')



"""
    "reversion_compare
    Add compare view to django-reversion for comparing two versions of a reversion model.
"""

__version__ = '0.16.0rc0'
__author__ = 'Jens Diemer <django-reversion-compare@jensdiemer.de>'



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2022 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from override_storage import locmem_stats_override_storage
from override_storage.utils import Stats
from reversion import create_revision, is_registered
from reversion.models import Revision, Version

from reversion_compare_project.models import VariantModel
from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class VariantModelNoDataTest(BaseTestCase):
    """
    Tests with a empty VariantModel
    """

    def test_textfield(self):
        with create_revision():
            item = VariantModel.objects.create(
                text="""\
first line
Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut
labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris
nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit
esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in
culpa qui officia deserunt mollit anim id est laborum.
last line"""
            )
            item.save()

        with create_revision():
            item.text = """\
first line
Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut
labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris
nisi ut aliquip ex ea commodo consequat. Duis added aute irure dolor in reprehenderit in voluptate velit
esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in
culpa qui officia deserunt mollit anim id est laborum.
last line"""
            item.save()

        response = self.client.get(
            "/en/admin/reversion_compare_project/variantmodel/1/history/compare/",
            data={"version_id2": 1, "version_id1": 2},
        )
        self.assertEqual(response.status_code, 200, response)
        self.assertContains(response, "<ins>added </ins>", msg_prefix=response.content.decode())


@locmem_stats_override_storage(name='storage_stats')
class VariantModelWithDataTest(BaseTestCase):
    """
    Tests with VariantModel and existing data from Fixtures()
    """

    storage_stats: Stats = None

    def setUp(self):
        super().setUp()

        self.item, self.fixtures = Fixtures(verbose=False).create_VariantModel_data()

        assert dict(self.storage_stats.saves_by_field) == {
            ('reversion_compare_project', 'variantmodel', 'file_field'): 'file_field_18.txt'
        }
        assert self.storage_stats.save_cnt == 20

        queryset = Version.objects.get_for_object(self.item)
        self.version_ids = queryset.values_list("pk", flat=True)

    def test_initial_state(self):
        self.assertTrue(is_registered(VariantModel))

        self.assertEqual(VariantModel.objects.count(), 1)

        count = len(self.fixtures) + 1  # incl. initial

        self.assertEqual(Version.objects.get_for_object(self.item).count(), count)
        self.assertEqual(Revision.objects.all().count(), count)
        self.assertEqual(len(self.version_ids), count)

    def test_all_changes(self):
        # debug_response(self.client.get("/admin/reversion_compare_project/variantmodel/1/history/"))
        # compare initial with last version
        response = self.client.get(
            "/en/admin/reversion_compare_project/variantmodel/1/history/compare/",
            data={"version_id2": 1, "version_id1": len(self.fixtures) + 1},  # incl. initial
        )
        self.assertEqual(response.status_code, 200, response)

        field_headlines = [f"<h3>{field_name.replace('_', ' ')}</h3>" for field_name, value in self.fixtures]
        self.assertContainsHtml(response, *field_headlines)
        self.assertContainsHtml(
            response,
            "<h3>boolean</h3>",
            '<p class="highlight"><del>False</del> changed to: <ins>True</ins></p>',
            "<h3>null boolean</h3>",
            '<p class="highlight"><del>None</del> changed to: <ins>False</ins></p>',
            "<h3>char</h3>",
            "<del>- a</del>",
            "<ins>+ B</ins>",
            "<h3>choices char</h3>",
            "<del>- alpha</del>",
            "<ins>+ bravo</ins>",
            "<h3>text</h3>",
            "<del>- Foo &#39;one&#39;</del>",
            "<ins>+ Bar &#39;two&#39;</ins>",
            "<h3>integer</h3>",
            "<del>- 0</del>",
            "<ins>+ -1</ins>",
            "<h3>integers</h3>",
            "<del>- 1,2,3</del>",
            "<ins>+ 2,3,4</ins>",
            "<h3>positive integer</h3>",
            "<del>- 1</del>",
            "<ins>+ 3</ins>",
            "<h3>big integer</h3>",
            "<del>- -9223372036854775808</del>",
            "<ins>+ 9223372036854775807</ins>",
            "<h3>time</h3>",
            "<del>- 20:15:00</del>",
            "<ins>+ 19:30:00</ins>",
            "<h3>date</h3>",
            "<del>- 1941-05-12</del>",
            "<ins>+ 2099-12-31</ins>",
            "<h3>datetime</h3>",
            "<del>- Aug. 19, 2005, 8:13 a.m.</del>",
            "<ins>+ Jan. 1, 2000, midnight</ins>",
            "<h3>decimal</h3>",
            "<del>- 1.23456789</del>",
            "<ins>+ 3.1415926535</ins>",
            "<h3>float</h3>",
            "<del>- 2.345</del>",
            "<ins>+ 3.1415</ins>",
            "<h3>email</h3>",
            "<del>- one@foo-bar.com</del>",
            "<ins>+ two@foo-bar.com</ins>",
            "<h3>url</h3>",
            """
            <div class="module">
                <pre class="highlight">
                <span class="diff-line diff-del diff-ins">http<ins>s</ins>://
                <del>www.pylucid.org</del><ins>github.com/jedie</ins>/</span>
                </pre>
            </div>
            """,
            "<h3>file field</h3>",
            """
            <div class="module">
            <pre class="highlight">
            <span class="diff-line diff-ins">/media/file_field<ins>_18</ins>.txt</span>
            </pre>
            </div>
            """,
            "<h3>filepath</h3>",
            """
            <div class="module">
            <pre class="highlight"><del>- foo/</del>
            <ins>+ bar/</ins></pre>
            </div>
            """,
            "<h3>ip address</h3>",
            "<del>- 192.168.0.1</del>",
            "<ins>+ 10.0.0.0</ins>",
        )



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2020 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from reversion.models import Version

from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class TemplateFieldModelTest(BaseTestCase):
    """
    unittests that used reversion_compare_test_app.models.SimpleModel

    Tests for the basic functions.
    """

    def setUp(self):
        super().setUp()
        fixtures = Fixtures(verbose=False)
        self.item1, self.item2 = fixtures.create_TemplateField_data()

        queryset = Version.objects.get_for_object(self.item1)
        self.version_ids1 = queryset.values_list("pk", flat=True)

        queryset = Version.objects.get_for_object(self.item2)
        self.version_ids2 = queryset.values_list("pk", flat=True)

    def test_diff(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/templatefield/{self.item1.pk}/history/compare/",
            data={"version_id2": self.version_ids1[0], "version_id1": self.version_ids1[1]},
        )
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            "<del>- version one</del>",
            "<ins>+ version two</ins>",
            "<blockquote>simply change the CharField text.</blockquote>",  # edit comment
        )



"""
    django-reversion helpers
    ~~~~~~~~~~~~~~~~~~~~~~~~

    A number of useful helper functions to automate common tasks.

    Used google-diff-match-patch [1] if installed, fallback to difflib.
    For installing use e.g. the unofficial package:

        pip install diff-match-patch

    [1] https://github.com/google/diff-match-patch

    :copyleft: 2012-2020 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import difflib
import logging

from diff_match_patch import diff_match_patch
from django.contrib import admin
from django.contrib.admin.sites import NotRegistered
from django.utils.encoding import force_str
from django.utils.html import escape
from django.utils.safestring import mark_safe


logger = logging.getLogger(__name__)


SEMANTIC = 1
EFFICIENCY = 2

# Change from diff-match-patch to ndiff if old/new values are less than X characters:
CHANGE_DIFF_THRESHOLD = 20


# https://github.com/google/diff-match-patch
dmp = diff_match_patch()


def highlight_diff(diff_text):
    """
    Simple highlight a diff text in the way pygments do it ;)
    """
    lines = []
    for line in diff_text.splitlines():
        line = escape(line)

        if line.startswith("+"):
            line = f"<ins>{line}</ins>"
        elif line.startswith("-"):
            line = f"<del>{line}</del>"

        lines.append(line)

    html = '\n'.join(lines)
    return f'<pre class="highlight">{html}</pre>'


def diff2lines(diff):
    """
    Group a sequence of diff_match_patch diff operations based on the
    lines they affect, so we can generate a nice-looking HTML diff.

    Example input:
        [(DIFF_EQUAL, "equal\ntext"),
         (DIFF_DELETE, "deleted\n"),
         (DIFF_INSERT, "added\ntext")]

    Example output:
        [(DIFF_EQUAL, "equal")],
        [(DIFF_EQUAL, "text"), (DIFF_DELETE, "deleted")],
        [(DIFF_INSERT, "added")],
        [(DIFF_INSERT, "text")],
    """
    curr_line = []
    for op, data in diff:
        data = escape(data)
        for line in data.splitlines(keepends=True):
            curr_line.append((op, line.rstrip("\r\n")))
            if line.endswith("\n"):
                yield curr_line
                curr_line = []
    if curr_line:
        yield curr_line


def lines2html(lines):
    """
    Convert a sequence of diff operations grouped by line (via diff2lines())
    to HTML for the diff viewer.

    Example input:
        [(DIFF_EQUAL, "equal")],
        [(DIFF_EQUAL, "text"), (DIFF_DELETE, "deleted")],
        [(DIFF_INSERT, "added")],
        [(DIFF_INSERT, "text")],

    Example output:
        'equal\n'
        '<span class="diff-line diff-del">text<del>deleted</del><del>⏎</del></span>\n'
        '<span class="diff-line diff-ins"><ins>added</ins></span>\n'
        '<span class="diff-line diff-del diff-ins"><ins>text</ins><del>removed</del></span>\n'
    """
    html = []

    for diff in lines:
        line = ''
        line_changes = set()
        for op, data in diff:
            if op == diff_match_patch.DIFF_EQUAL:
                line += data
            elif op == diff_match_patch.DIFF_INSERT:
                line += f'<ins>{data or "⏎"}</ins>'
                line_changes.add('ins')
            elif op == diff_match_patch.DIFF_DELETE:
                line += f'<del>{data or "⏎"}</del>'
                line_changes.add('del')
            else:
                raise TypeError(f'Unknown diff op: {op!r}')
        if line_changes:
            classes = ' '.join(f'diff-{change_type}' for change_type in sorted(line_changes))
            html.append(f'<span class="diff-line {classes}">{line}</span>\n')
        else:
            html.append(f'{line}\n')

    return ''.join(html)


def diff_match_patch_pretty_html(diff):
    """
    Similar to diff_match_patch.diff_prettyHtml but generated the same html as our
    reversion_compare.helpers.highlight_diff
    """
    html = ['<pre class="highlight">']
    html.extend(lines2html(diff2lines(diff)))
    html.append("</pre>")
    return "".join(html)


def generate_dmp_diff(value1, value2, cleanup=SEMANTIC):
    """
    Generate the diff with Google diff-match-patch
    """
    diff = dmp.diff_main(
        value1, value2,
        checklines=True  # run a line-level diff first to identify the changed areas
    )
    if cleanup == SEMANTIC:
        dmp.diff_cleanupSemantic(diff)
    elif cleanup == EFFICIENCY:
        dmp.diff_cleanupEfficiency(diff)
    elif cleanup is not None:
        raise ValueError("cleanup parameter should be one of SEMANTIC, EFFICIENCY or None.")

    html = diff_match_patch_pretty_html(diff)

    return html


def generate_ndiff(value1, value2):
    value1 = value1.splitlines()
    value2 = value2.splitlines()
    diff = difflib.ndiff(value1, value2)
    diff_text = "\n".join(diff)
    html = highlight_diff(diff_text)
    return html


def html_diff(value1, value2, cleanup=SEMANTIC):
    """
    Generates a diff used google-diff-match-patch is exist or ndiff as fallback

    The cleanup parameter can be SEMANTIC, EFFICIENCY or None to clean up the diff
    for greater human readibility.
    """
    value1 = force_str(value1, errors='replace')
    value2 = force_str(value2, errors='replace')

    if len(value1) > CHANGE_DIFF_THRESHOLD or len(value2) > CHANGE_DIFF_THRESHOLD:
        # Bigger values -> use Google diff-match-patch
        html = generate_dmp_diff(value1, value2, cleanup)
    else:
        # For small content use ndiff
        html = generate_ndiff(value1, value2)

    html = mark_safe(html)
    return html


def compare_queryset(first, second):
    """
    Simple compare two querysets (used for many-to-many field compare)
    XXX: resort results?
    """
    result = []
    for item in set(first).union(set(second)):
        if item not in first:  # item was inserted
            item.insert = True
        elif item not in second:  # item was deleted
            item.delete = True
        result.append(item)
    return result


def patch_admin(model, admin_site=None, AdminClass=None, skip_non_revision=False):
    """
    Enables version control with full admin integration for a model that has
    already been registered with the django admin site.

    This is excellent for adding version control to existing Django contrib
    applications.

    :param skip_non_revision: If ==True: Skip models that are not register with ModelAdmin
    """
    admin_site = admin_site or admin.site
    try:
        ModelAdmin = admin_site._registry[model].__class__
    except KeyError:
        raise NotRegistered(f"The model {model} has not been registered with the admin site.")

    if skip_non_revision:
        if not hasattr(ModelAdmin, "object_history_template"):
            logger.info(
                f"Skip activate compare admin, because"
                f" model {model._meta.object_name!r} is not registered with revision manager."
            )
        return

    # Unregister existing admin class.
    admin_site.unregister(model)

    # Register patched admin class.
    if not AdminClass:
        from reversion_compare.admin import CompareVersionAdmin

        class PatchedModelAdmin(CompareVersionAdmin, ModelAdmin):
            pass

    else:

        class PatchedModelAdmin(AdminClass, ModelAdmin):
            pass

    admin_site.register(model, PatchedModelAdmin)



# Generated by Django 4.1.8 on 2023-04-07 10:05

import re

import django.core.validators
import django.db.models.deletion
import django_countries.fields
import django_tools.file_storage.file_system_storage
from django.db import migrations, models

import reversion_compare_project.models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Building',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('address', models.CharField(max_length=128)),
            ],
        ),
        migrations.CreateModel(
            name='CountryFieldTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('one_country', django_countries.fields.CountryField(max_length=2)),
                ('multiple_countries', django_countries.fields.CountryField(max_length=746, multiple=True)),
            ],
        ),
        migrations.CreateModel(
            name='CustomModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='MigrationModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'info',
                    models.CharField(
                        help_text='This field never changed and contains info about migration/version', max_length=255
                    ),
                ),
                (
                    'number_then_text',
                    models.IntegerField(help_text='migrate between CharField -> IntegerField -> CharField'),
                ),
                (
                    'text',
                    models.CharField(
                        help_text='TextField -> CharField(max_length=128) -> CharField(max_length=64)', max_length=128
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Pet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='SimpleModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='VariantModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('boolean', models.BooleanField(default=True)),
                ('null_boolean', models.BooleanField(null=True)),
                ('char', models.CharField(blank=True, max_length=1, null=True)),
                (
                    'choices_char',
                    models.CharField(blank=True, choices=[('a', 'alpha'), ('b', 'bravo')], max_length=1, null=True),
                ),
                ('text', models.TextField(blank=True, null=True)),
                ('integer', models.IntegerField(blank=True, null=True)),
                (
                    'integers',
                    models.CharField(
                        blank=True,
                        max_length=64,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                re.compile('^\\d+(?:,\\d+)*\\Z'),
                                code='invalid',
                                message='Enter only digits separated by commas.',
                            )
                        ],
                    ),
                ),
                ('positive_integer', models.PositiveIntegerField(blank=True, null=True)),
                ('big_integer', models.BigIntegerField(blank=True, null=True)),
                ('time', models.TimeField(blank=True, null=True)),
                ('date', models.DateField(blank=True, null=True)),
                ('datetime', models.DateTimeField(blank=True, null=True)),
                ('decimal', models.DecimalField(blank=True, decimal_places=3, max_digits=5, null=True)),
                ('float', models.FloatField(blank=True, null=True)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('url', models.URLField(blank=True, null=True)),
                (
                    'file_field',
                    models.FileField(
                        blank=True,
                        null=True,
                        storage=django_tools.file_storage.file_system_storage.OverwriteFileSystemStorage(
                            create_backups=False
                        ),
                        upload_to='',
                    ),
                ),
                (
                    'filepath',
                    models.FilePathField(blank=True, null=True, path=reversion_compare_project.models.get_filepath),
                ),
                ('ip_address', models.GenericIPAddressField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Factory',
            fields=[
                (
                    'building_ptr',
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to='reversion_compare_project.building',
                    ),
                ),
                ('name', models.CharField(max_length=128)),
            ],
            bases=('reversion_compare_project.building',),
        ),
        migrations.CreateModel(
            name='TemplateField',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.CharField(max_length=20)),
                (
                    'template',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to='reversion_compare_project.person',
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name='person',
            name='pets',
            field=models.ManyToManyField(blank=True, to='reversion_compare_project.pet'),
        ),
        migrations.AddField(
            model_name='person',
            name='workplace',
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name='workers',
                to='reversion_compare_project.building',
            ),
        ),
        migrations.CreateModel(
            name='Identity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('id_numer', models.CharField(max_length=100)),
                (
                    'person',
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='_identity',
                        to='reversion_compare_project.person',
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='Car',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
                (
                    'manufacturer',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='cars',
                        to='reversion_compare_project.factory',
                    ),
                ),
                (
                    'supplier',
                    models.ManyToManyField(
                        blank=True, related_name='suppliers', to='reversion_compare_project.factory'
                    ),
                ),
            ],
        ),
    ]



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2022 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import datetime
import json
from decimal import Decimal

from bx_py_utils.test_utils.datetime import parse_dt
from django.contrib.contenttypes.models import ContentType
from django.core.files.base import ContentFile
from django.db.models import BigIntegerField
from reversion import create_revision, set_comment
from reversion.models import Revision, Version

from reversion_compare_project.models import (
    Car,
    CountryFieldTestModel,
    CustomModel,
    Factory,
    Identity,
    MigrationModel,
    Person,
    Pet,
    SimpleModel,
    TemplateField,
    VariantModel,
)


LOREM_IPSUM = (
    'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor'
    ' invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et'
    ' accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata'
    ' sanctus est Lorem ipsum dolor sit amet. Lorem ipsum dolor sit amet, consetetur sadipscing'
    ' elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat,'
    ' sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita'
    ' kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.'
)


class Fixtures:
    """
    Collection of all test data creation method.
    This will be also used from external scripts, too!
    """

    def __init__(self, verbose=False):
        self.verbose = verbose

    def create_all(self):
        """
        simple call all create_*_data() methods
        """
        for method_name in dir(self):
            if method_name.startswith("create_") and method_name.endswith("_data"):
                if self.verbose:
                    print("_" * 79)
                    print(f" *** {method_name} ***")
                func = getattr(self, method_name)
                func()

    def create_Simple_data(self):
        with create_revision():
            item1 = SimpleModel.objects.create(text="version one")

        if self.verbose:
            print("version 1:", item1)

        with create_revision():
            item1.text = "version two"
            item1.save()
            set_comment("simply change the CharField text.")

        if self.verbose:
            print("version 2:", item1)

        for no in range(5):
            with create_revision():
                if no == 0:
                    item2 = SimpleModel.objects.create(text="v0")
                    set_comment(f"create v{no:d}")
                else:
                    item2.text = f"v{no:d}"
                    item2.save()
                    set_comment(f"change to v{no:d}")

        return item1, item2

    def create_FactoryCar_data(self):
        with create_revision():
            manufacture = Factory.objects.create(name="factory one", address="1 Fake Plaza")
            supplier1 = Factory.objects.create(name="always the same supplier", address="1 Fake Plaza")
            supplier2 = Factory.objects.create(name="would be deleted supplier", address="1 Fake Plaza")
            supplier3 = Factory.objects.create(name="would be removed supplier", address="1 Fake Plaza")
            car = Car.objects.create(name="motor-car one", manufacturer=manufacture)
            car.supplier.add(supplier1, supplier2, supplier3)
            car.save()
            set_comment("initial version 1")

        if self.verbose:
            print("version 1:", car)
            # motor-car one from factory one supplier(s): always the same supplier,
            # would be deleted supplier, would be removed supplier

        """ 1 to 2 diff:

        "manufacture" ForeignKey:
            "factory one" -> "factory I"

        "supplier" ManyToManyField:
            + new, would be renamed supplier
            - would be deleted supplier
            - would be removed supplier
            = always the same supplier
        """

        with create_revision():
            manufacture.name = "factory I"
            manufacture.save()
            supplier2.delete()  # - would be deleted supplier
            supplier4 = Factory.objects.create(name="new, would be renamed supplier", address="1 Fake Plaza")
            car.supplier.add(supplier4)  # + new, would be renamed supplier
            car.supplier.remove(supplier3)  # - would be removed supplier
            car.save()
            set_comment("version 2: change ForeignKey and ManyToManyField.")

        if self.verbose:
            print("version 2:", car)
            # motor-car one from factory I supplier(s): always the same supplier, new, would be renamed supplier

        """ 2 to 3 diff:

        "name" CharField:
            "motor-car one" -> "motor-car II"

        "manufacture" ForeignKey:
            "factory I" -> "factory II"

        "supplier" ManyToManyField:
            new, would be renamed supplier -> not new anymore supplier
            = always the same supplier
        """

        with create_revision():
            car.name = "motor-car II"
            manufacture.name = "factory II"
            supplier4.name = "not new anymore supplier"
            supplier4.save()
            car.save()
            set_comment("version 3: change CharField, ForeignKey and ManyToManyField.")

        if self.verbose:
            print("version 3:", car)
            # version 3: motor-car II from factory II supplier(s): always the same supplier, not new anymore supplier

        return car

    def create_FactoryCar_fk_change_data(self):
        with create_revision():
            manufacturer = Factory.objects.create(name="factory one", address="1 Fake Plaza")
            different_manufacturer = Factory.objects.create(name="factory two", address="1 Fake Plaza")
            car = Car.objects.create(name="motor-car one", manufacturer=manufacturer)
            car.save()
            set_comment("initial version 1")

        if self.verbose:
            print("version 1:", car)

        with create_revision():
            car.name = "motor-car two"
            car.save()
            manufacturer.name = "factory I"
            manufacturer.save()

        if self.verbose:
            print("version 2:", car)

        with create_revision():
            car.manufacturer = different_manufacturer
            car.save()

        if self.verbose:
            print("version 3:", car)

        return car

    def create_Factory_reverse_relation_data(self):
        from django.db import transaction

        with transaction.atomic(), create_revision():
            manufacturer = Factory.objects.create(name="factory one", address="1 Fake Plaza")
            different_manufacturer = Factory.objects.create(name="factory two", address="1 Fake Plaza")
            car1 = Car.objects.create(name="motor-car one", manufacturer=manufacturer)
            car2 = Car.objects.create(name="motor-car two", manufacturer=manufacturer)
            car3 = Car.objects.create(name="motor-car three", manufacturer=manufacturer)
            car1.save()
            car2.save()
            car3.save()
            manufacturer.save()
            set_comment("initial version 1")

        if self.verbose:
            print("version 1:", manufacturer)
            # Factory One

        """ 1 to 2 diff:

        "manufacture" ForeignKey:
            "factory one" -> "factory I"

        "supplier" ManyToManyField:
            + new, would be renamed supplier
            - would be deleted supplier
            - would be removed supplier
            = always the same supplier
        """

        with transaction.atomic(), create_revision():
            car3.delete()
            car4 = Car.objects.create(name="motor-car four", manufacturer=manufacturer)
            car4.save()

            worker1 = Person.objects.create(name="Bob Bobertson", workplace=manufacturer)
            worker1.save()

            manufacturer.save()
            set_comment("version 2: discontinued car-three, add car-four, add Bob the worker")

        if self.verbose:
            print("version 2:", manufacturer)
            # motor-car one from factory I supplier(s): always the same supplier, new, would be renamed supplier

        """ 2 to 3 diff:

        "name" CharField:
            "motor-car one" -> "motor-car II"

        "manufacture" ForeignKey:
            "factory I" -> "factory II"

        "supplier" ManyToManyField:
            new, would be renamed supplier -> not new anymore supplier
            = always the same supplier
        """

        with transaction.atomic(), create_revision():
            car2.manufacturer = different_manufacturer
            car2.save()
            manufacturer.save()
            set_comment("version 3: car2 now built by someone else.")

        if self.verbose:
            print("version 3:", manufacturer)
            # version 3: motor-car II from factory II supplier(s): always the same supplier, not new anymore supplier

        return manufacturer

    def create_PersonPet_data(self):
        with create_revision():
            pet1 = Pet.objects.create(name="would be changed pet")
            pet2 = Pet.objects.create(name="would be deleted pet")
            pet3 = Pet.objects.create(name="would be removed pet")
            pet4 = Pet.objects.create(name="always the same pet")
            person = Person.objects.create(name="Dave")
            person.pets.add(pet1, pet2, pet3, pet4)
            person.save()
            set_comment("initial version 1")

        if self.verbose:
            print("version 1:", person, person.pets.all())
            # Dave [<Pet: would be changed pet>, <Pet: would be deleted pet>, <Pet:
            # would be removed pet>, <Pet: always the same pet>]

        """ 1 to 2 diff:

        "pets" ManyToManyField:
            would be changed pet -> Is changed pet
            - would be removed pet
            - would be deleted pet
            = always the same pet
        """

        with create_revision():
            pet1.name = "Is changed pet"
            pet1.save()
            pet2.delete()
            person.pets.remove(pet3)
            person.save()
            set_comment("version 2: change follow related pets.")

        if self.verbose:
            print("version 2:", person, person.pets.all())
            # Dave [<Pet: Is changed pet>, <Pet: always the same pet>]

        return pet1, pet2, person

    def create_VariantModel_data(self):
        with create_revision():
            item = VariantModel(
                boolean=False,
                null_boolean=None,
                char="a",
                choices_char="a",
                text="Foo 'one'",
                # skip: models.SlugField()
                integer=0,
                integers="1,2,3",  # CommaSeparatedIntegerField
                positive_integer=1,
                big_integer=(-BigIntegerField.MAX_BIGINT - 1),
                # skip:
                # models.PositiveSmallIntegerField()
                # models.SmallIntegerField()
                time=datetime.time(hour=20, minute=15),
                date=datetime.date(year=1941, month=5, day=12),  # Z3 was presented in germany ;)
                # PyLucid v0.0.1 release date:
                datetime=datetime.datetime(year=2005, month=8, day=19, hour=8, minute=13, second=24),
                decimal=Decimal("1.23456789"),
                float=2.345,
                email="one@foo-bar.com",
                url="http://www.pylucid.org/",
                filepath='foo/',
                ip_address="192.168.0.1",
                # skip: models.GenericIPAddressField()
            )
            item.file_field.save('file_field.txt', ContentFile('file_field_content'), save=False)
            # TODO: item.full_clean()
            item.save()
            set_comment("initial version")

        fixtures = (
            ("boolean", True),
            ("null_boolean", True),
            ("null_boolean", False),
            ("char", "B"),
            ("choices_char", "b"),
            ("text", "Bar 'two'"),
            # skip: models.SlugField()
            ("integer", -1),
            ("integers", "2,3,4"),  # CommaSeparatedIntegerField
            ("positive_integer", 3),
            ("big_integer", BigIntegerField.MAX_BIGINT),
            # models.PositiveSmallIntegerField()
            # models.SmallIntegerField()
            ("time", datetime.time(hour=19, minute=30)),
            ("date", datetime.date(year=2099, month=12, day=31)),
            ("datetime", datetime.datetime(year=2000, month=1, day=1, hour=0, minute=0, second=1)),
            ("decimal", Decimal("3.1415926535")),
            ("float", 3.1415),
            ("email", "two@foo-bar.com"),
            ("url", "https://github.com/jedie/"),
            ("filepath", "bar/"),
            ("ip_address", "10.0.0.0"),
        )
        for no, (field_name, value) in enumerate(fixtures):
            with create_revision():
                setattr(item, field_name, value)
                item.file_field.save(
                    f'file_field_{no}.txt',
                    ContentFile('file_field_content_{no}'),
                    save=False,
                )
                item.save()
                set_comment(f"{no:d} change: {field_name!r} field.")

        return item, fixtures

    def create_CustomModel_data(self):
        with create_revision():
            item1 = CustomModel.objects.create(text="version one")

        if self.verbose:
            print("version 1:", item1)

        return item1

    def create_PersonIdentity_data(self):
        with create_revision():
            person = Person.objects.create(name="Dave")
            identity = Identity.objects.create(id_numer="1234", person=person)

        if self.verbose:
            print("version 1:", person, identity)

        with create_revision():
            person.name = "John"
            person.save()
            set_comment("version 2: change person name.")

        return person, identity

    def create_TemplateField_data(self):
        with create_revision():
            item1 = TemplateField.objects.create(text="version one")

        if self.verbose:
            print("version 1:", item1)

        with create_revision():
            item1.text = "version two"
            item1.save()
            set_comment("simply change the CharField text.")

        if self.verbose:
            print("version 2:", item1)

        for no in range(5):
            with create_revision():
                if no == 0:
                    item2 = TemplateField.objects.create(text="v0")
                    set_comment(f"create v{no:d}")
                else:
                    item2.text = f"v{no:d}"
                    item2.save()
                    set_comment(f"change to v{no:d}")

        return item1, item2

    def create_MigrationModel_data(self):
        """
        Called only for tests!
        Normal run creates example data via migrations:
            reversion_compare_project/migrations/0002_migration_model_1.py
            reversion_compare_project/migrations/0003_migration_model_2.py

        Generate the same reversions here as in migrations.
        """
        if MigrationModel.objects.count():
            # Skip creation called by 'make run-test-server'
            return

        instance = MigrationModel.objects.create(  # last version
            pk=1,
            info='Migration state 2 - version 4',
            number_then_text=111,
            text='Now this is a short text!!!',
        )
        content_type = ContentType.objects.get_for_model(instance)

        def create_version(date, comment, data):
            Version.objects.create(
                revision=Revision.objects.create(date_created=date, comment=comment),
                object_id=1,
                content_type=content_type,
                db='default',
                format='json',
                serialized_data=json.dumps(
                    [{'model': 'reversion_compare_project.migrationmodel', 'pk': 1, 'fields': data}]
                ),
                object_repr='MigrationModel object (1)',
            )

        create_version(
            date=parse_dt('2020-01-01T12:00:00+0000'),
            comment='Migration state 1 - version 1',
            data={'info': 'Migration state 1 - version 1', 'number_then_text': 'Not a number 1', 'text': LOREM_IPSUM},
        )
        create_version(
            date=parse_dt('2020-01-02T12:00:00+0000'),
            comment='Migration state 1 - version 2',
            data={'info': 'Migration state 1 - version 2', 'number_then_text': 'Not a number 2', 'text': LOREM_IPSUM},
        )
        create_version(
            date=parse_dt('2020-01-03T12:00:00+0000'),
            comment='Migration state 2 - version 3',
            data={
                'info': 'Migration state 2 - version 3',
                'number_then_text': 789,
                'text': 'Now this is a short text.',
            },
        )
        create_version(
            date=parse_dt('2020-01-04T12:00:00+0000'),
            comment='Migration state 2 - version 4',
            data={
                'info': 'Migration state 2 - version 4',
                'number_then_text': 111,
                'text': 'Now this is a short text!!!',
            },
        )

        return instance

    def create_CountryFieldTestModel_data(self):
        with create_revision():
            item = CountryFieldTestModel.objects.create(
                one_country='de',
                multiple_countries=sorted(['de', 'at']),
            )
            set_comment('init')

        if self.verbose:
            print("version 1:", item)

        with create_revision():
            item.one_country = 'gb'
            item.multiple_countries = sorted(['de', 'gb', 'ch', 'at'])
            item.save()
            set_comment('Change')

        if self.verbose:
            print("version 2:", item)

        return item



"""
    Allow reversion_compare to be executable
    through `python -m reversion_compare`.
"""
from manage_django_project.manage import execute_django_from_command_line


def main():
    """
    entrypoint installed via pyproject.toml and [project.scripts] section.
    Must be set in ./manage.py and PROJECT_SHELL_SCRIPT
    """
    execute_django_from_command_line()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2016 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from reversion import create_revision, is_registered
from reversion.models import Revision, Version

from reversion_compare_project.models import Car, Factory
from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class FactoryCarModelTest(BaseTestCase):
    """
    unittests that used:
        reversion_compare_test_app.models.Factory
        reversion_compare_test_app.models.Car

    Factory & Car would be registered only in admin.py
    so no relation data would be stored
    """

    def setUp(self):
        super().setUp()

        fixtures = Fixtures(verbose=False)
        # fixtures = Fixtures(verbose=True)
        self.car = fixtures.create_FactoryCar_data()

        queryset = Version.objects.get_for_object(self.car)
        self.version_ids = queryset.values_list("pk", flat=True)

    def test_initial_state(self):
        self.assertTrue(is_registered(Factory))
        self.assertTrue(is_registered(Car))

        self.assertEqual(Revision.objects.all().count(), 3)
        self.assertEqual(len(self.version_ids), 3)
        self.assertEqual(Version.objects.all().count(), 17)

    def test_select_compare(self):
        response = self.client.get(f"/en/admin/reversion_compare_project/car/{self.car.pk}/history/")
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{self.version_ids[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{self.version_ids[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[1]:d}" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[2]:d}" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[2]:d}" />',
        )

    def test_diff1(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/car/{self.car.pk}/history/compare/",
            data={"version_id2": self.version_ids[1], "version_id1": self.version_ids[2]},
        )
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            '<h3>manufacturer<sup class="follow">*</sup></h3>',
            '<h3>supplier<sup class="follow">*</sup></h3>',
            """
            <p class="highlight">
                <del>- would be deleted supplier</del><br />
                <del>- would be removed supplier</del><br />
                <ins>+ new, would be renamed supplier</ins><br />
                always the same supplier<sup class="follow">*</sup><br />
            </p>
            """,
            '<h4 class="follow">Note:</h4>',  # info for non-follow related informations
            "<blockquote>version 2: change ForeignKey and ManyToManyField.</blockquote>",
        )

    def test_diff2(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/car/{self.car.pk}/history/compare/",
            data={"version_id2": self.version_ids[0], "version_id1": self.version_ids[1]},
        )
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            "<del>- motor-car one</del>",
            "<ins>+ motor-car II</ins>",
            '<h3>manufacturer<sup class="follow">*</sup></h3>',
            '<h3>supplier<sup class="follow">*</sup></h3>',
            """
            <p class="highlight">
                <del>new, would be renamed supplier</del> &rarr; <ins>not new anymore supplier</ins><br />
                always the same supplier<sup class="follow">*</sup><br />
            </p>
            """,
            '<h4 class="follow">Note:</h4>',  # info for non-follow related informations
            "<blockquote>version 3: change CharField, ForeignKey and ManyToManyField.</blockquote>",
        )


class FactoryCarModelTest2(BaseTestCase):
    """
    unittests that used:
        reversion_compare_test_app.models.Factory
        reversion_compare_test_app.models.Car

    Factory & Car would be registered only in admin.py
    so no relation data would be stored
    """

    def test_initial_state(self):
        self.assertTrue(is_registered(Factory))
        self.assertTrue(is_registered(Car))

        self.assertEqual(Factory.objects.all().count(), 0)
        self.assertEqual(Car.objects.all().count(), 0)

        self.assertEqual(Revision.objects.all().count(), 0)
        self.assertEqual(Version.objects.all().count(), 0)

    def test_deleted_objects(self):
        """
        for:
        https://github.com/jedie/django-reversion-compare/commit/ba22008130f4c16a32eeb900381c2d82ca6fdd9e
        https://travis-ci.org/jedie/django-reversion-compare/builds/72317520
        """
        with create_revision():
            factory1 = Factory.objects.create(name="factory one", address="1 Fake Plaza")
            car = Car.objects.create(name="car", manufacturer=factory1)

        with create_revision():
            factory2 = Factory.objects.create(name="factory two", address="1 Fake Plaza")
            car.manufacturer = factory2
            car.save()

        with create_revision():
            factory1.delete()

        queryset = Version.objects.get_for_object(car)
        version_ids = queryset.values_list("pk", flat=True)  # [3, 2]
        # print("\n", version_ids)

        # response = self.client.get("/admin/reversion_compare_project/car/%s/history/" % car.pk)
        # debug_response(response) # from django-tools
        self.client.get(
            f"/en/admin/reversion_compare_project/car/{car.pk}/history/compare/",
            data={"version_id2": version_ids[0], "version_id1": version_ids[1]},
        )
        # debug_response(response) # from django-tools


class FactoryCarModelTest3(BaseTestCase):
    """
    unittests that used:
        reversion_compare_test_app.models.Factory
        reversion_compare_test_app.models.Car

    Factory & Car would be registered only in admin.py
    so no relation data would be stored
    """

    def setUp(self):
        super().setUp()

        fixtures = Fixtures(verbose=False)
        # fixtures = Fixtures(verbose=True)
        self.car = fixtures.create_FactoryCar_fk_change_data()

        queryset = Version.objects.get_for_object(self.car).order_by("pk")
        self.version_ids = queryset.values_list("pk", flat=True)

    def test_initial_state(self):
        self.assertTrue(is_registered(Factory))
        self.assertTrue(is_registered(Car))

        self.assertEqual(Revision.objects.all().count(), 3)
        self.assertEqual(len(self.version_ids), 3)
        self.assertEqual(Version.objects.all().count(), 9)

    def ftest_diff1(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/car/{self.car.pk}/history/compare/",
            data={"version_id2": self.version_ids[0], "version_id1": self.version_ids[1]},
        )
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            '<h3>manufacturer<sup class="follow">*</sup></h3>',
            '<p><span class="highlight">factory I</span></p>',
        )

    def test_diff1_fk_as_id(self):
        with self.settings(REVERSION_COMPARE_FOREIGN_OBJECTS_AS_ID=True):
            response = self.client.get(
                f"/en/admin/reversion_compare_project/car/{self.car.pk}/history/compare/",
                data={"version_id2": self.version_ids[0], "version_id1": self.version_ids[1]},
            )
            # debug_response(response) # from django-tools
            self.assertNotContainsHtml(
                response,
                '<h3>manufacturer<sup class="follow">*</sup></h3>',
                '<p><span class="highlight">factory I</span></p>',
            )

    def test_diff2(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/car/{self.car.pk}/history/compare/",
            data={"version_id2": self.version_ids[1], "version_id1": self.version_ids[2]},
        )
        # debug_response(response) # from django-tools
        self.assertContainsHtml(response, "<del>- factory I</del>", "<ins>+ factory two</ins>")
        self.assertContainsHtml(
            response,
            """
            <div class="module">
                <pre class="highlight">
                    <del>- factory I</del>
                    ?         ^

                    <ins>+ factory two</ins>
                    ?         ^^^
                </pre>
            </div>
        """,
        )

    def test_diff2_fk_as_id(self):
        with self.settings(REVERSION_COMPARE_FOREIGN_OBJECTS_AS_ID=True):
            response = self.client.get(
                f"/en/admin/reversion_compare_project/car/{self.car.pk}/history/compare/",
                data={"version_id2": self.version_ids[1], "version_id1": self.version_ids[2]},
            )
        # debug_response(response) # from django-tools
        self.assertContainsHtml(response, "<del>- factory I</del>", "<ins>+ factory two</ins>")



"""
    :copyleft: 2020 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import sys

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User


def _print_and_message(request, msg, level=messages.WARNING):
    print(f" *** {msg} ***", file=sys.stderr)
    messages.add_message(request, level, msg)


class AlwaysLoggedInAsSuperUser:
    """
    Auto login all users as default superuser.
    Default user will be created, if not exist.

    Disable it by deactivate the default user.
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        self._auto_login(request)
        response = self.get_response(request)

        return response

    def _auto_login(self, request):
        if request.user.is_authenticated:
            return

        try:
            user = User.objects.get(username=settings.DEFAULT_USERNAME)
        except User.DoesNotExist:
            _print_and_message(request, "Create default django user.")
            User.objects.create_superuser(
                settings.DEFAULT_USERNAME, "nobody@local.intranet", settings.DEFAULT_USERPASS
            )
        else:
            if not user.is_active:
                _print_and_message(request, "Default User was deactivated!", level=messages.ERROR)
                return

            user.set_password(settings.DEFAULT_USERPASS)
            user.save()

        _print_and_message(request, f"Autologin applyed. Your logged in as {settings.DEFAULT_USERNAME!r}")
        user = authenticate(username=settings.DEFAULT_USERNAME, password=settings.DEFAULT_USERPASS)
        login(request, user)



"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2017 by the django-reversion-compare team, see AUTHORS for more details.
    :created: 2017 by Jens Diemer <github.com@jensdiemer.de>
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import re


def print_db_queries(queries):
    queries_data = {}
    for query in queries:
        sql = query["sql"]
        queries_data.setdefault(sql, 0)
        queries_data[sql] += 1
    duplicates = sum([count - 1 for count in list(queries_data.values())])
    print("-" * 79)
    print(f"total queries....: {len(queries)}")
    print(f"unique queries...: {len(queries_data)}")
    print(f"duplicate queries: {duplicates:d}")
    print()
    for query, count in sorted(queries_data.items()):
        query = re.sub(r'["\'`]', "", query)
        print(f"{count} x {query}")
    print("-" * 79)
